@@ -18,6 +18,7 @@ import Prelude hiding (read, length)
 import Control.Monad.Primitive
 
 import Data.Vector.Generic.Mutable
+import Data.Word (Word)
 
 import qualified Data.Vector.Primitive.Mutable as PV
 
@@ -46,3 +47,13 @@ countLoop rdx src count = set count 0 >> go 0
    | otherwise  = return ()
 {-# INLINE countLoop #-}
 
+midPoint :: Int -> Int -> Int
+midPoint a b =
+  toInt $ (toWord a + toWord b) `div` 2
+  where
+    toWord :: Int -> Word
+    toWord = fromIntegral
+
+    toInt :: Word -> Int
+    toInt = fromIntegral
+{-# INLINE midPoint #-}
